#version 320

precision highp float;

layout(binding=0) uniform BlockFragment {
	vec4 pParam;
};

layout(location = 0, binding = 0) uniform sampler2D Sampler0;

in vec2 texCoord;

layout(location = 0) out vec4 pColor;

const vec2 TextureSize = vec2(320.0, 240.0);
const vec2 OutputSize  = vec2(1920.0, 1080.0);
const vec2 InputSize = vec2(320.0, 240.0);

#define COMPAT_TEXTURE texture
#define Source Sampler0
#define INTERPOLATE_IN_LINEAR_GAMMA 1.0

#define SourceSize vec4(TextureSize, 1.0 / TextureSize)
#define outsize vec4(OutputSize, 1.0 / OutputSize)


void main()
{

   vec2 texelSize = SourceSize.zw;

   vec2 range = vec2(abs(InputSize.x / (outsize.x * SourceSize.x)), abs(InputSize.y / (outsize.y * SourceSize.y)));
   range = range / 2.0 * 0.999;

   float left   = texCoord.x - range.x;
   float top    = texCoord.y + range.y;
   float right  = texCoord.x + range.x;
   float bottom = texCoord.y - range.y;
   
   vec3 topLeftColor     = COMPAT_TEXTURE(Source, (floor(vec2(left, top)     / texelSize) + 0.5) * texelSize).rgb;
   vec3 bottomRightColor = COMPAT_TEXTURE(Source, (floor(vec2(right, bottom) / texelSize) + 0.5) * texelSize).rgb;
   vec3 bottomLeftColor  = COMPAT_TEXTURE(Source, (floor(vec2(left, bottom)  / texelSize) + 0.5) * texelSize).rgb;
   vec3 topRightColor    = COMPAT_TEXTURE(Source, (floor(vec2(right, top)    / texelSize) + 0.5) * texelSize).rgb;

   if (INTERPOLATE_IN_LINEAR_GAMMA > 0.5){
	topLeftColor     = pow(topLeftColor, vec3(2.2));
	bottomRightColor = pow(bottomRightColor, vec3(2.2));
	bottomLeftColor  = pow(bottomLeftColor, vec3(2.2));
	topRightColor    = pow(topRightColor, vec3(2.2));
   }

   vec2 border = clamp(floor((texCoord / texelSize) + vec2(0.5)) * texelSize, vec2(left, bottom), vec2(right, top));

   float totalArea = 4.0 * range.x * range.y;

   vec3 averageColor;
   averageColor  = ((border.x - left)  * (top - border.y)    / totalArea) * topLeftColor;
   averageColor += ((right - border.x) * (border.y - bottom) / totalArea) * bottomRightColor;
   averageColor += ((border.x - left)  * (border.y - bottom) / totalArea) * bottomLeftColor;
   averageColor += ((right - border.x) * (top - border.y)    / totalArea) * topRightColor;

   pColor = (INTERPOLATE_IN_LINEAR_GAMMA > 0.5) ? vec4(pow(averageColor, vec3(1.0 / 2.2)), 1.0) : vec4(averageColor, 1.0);
}
