#version 320

precision highp float;

layout(binding=0) uniform BlockFragment {
	vec4 pParam1;
};

layout(location = 0, binding = 0) uniform sampler2D Sampler0;

in vec2 texCoord;
in vec4 t1;
in vec4 t2;
in vec4 t3;
in vec4 t4;
in vec4 t5;
in vec4 t6;
in vec4 t7;

layout(location = 0) out vec4 pColor;


vec3 res2x(vec3 pre2, vec3 pre1, vec3 px, vec3 pos1, vec3 pos2)
{
    vec3 t, m;
    mat4x3 pre = mat4x3(pre2, pre1,   px, pos1);
    mat4x3 pos = mat4x3(pre1,   px, pos1, pos2);
    mat4x3  df = pos - pre;

    m = 0.5 - abs(px - 0.5);
    m = 2.0 * min(m, min(abs(df[1]), abs(df[2])));
    t = (7.0 * (df[1] + df[2]) - 3.0 * (df[0] + df[3])) / 16.0;
    t = clamp(t, -m, m);

    return t;
}



void main()
{
	//vec3 dt = pParam1.xyz;

	vec2 fp = 2.0*fract(texCoord*pParam1.xy) - vec2(1.0, 1.0);
	
	vec3 B1  = texture(Sampler0, t1.yw).xyz; 
	vec3 B   = texture(Sampler0, t2.yw).xyz; 
	vec3 D   = texture(Sampler0, t3.xw).xyz; 
	vec3 E   = texture(Sampler0, t3.yw).xyz; 
	vec3 F   = texture(Sampler0, t3.zw).xyz; 
	vec3 H   = texture(Sampler0, t4.yw).xyz; 
	vec3 H5  = texture(Sampler0, t5.yw).xyz; 
	vec3 D0  = texture(Sampler0, t6.xz).xyz; 
	vec3 F4  = texture(Sampler0, t7.xz).xyz; 

	vec3 s1 = res2x(B1, B,  E, H, H5)*fp.y;
	vec3 s2 = res2x(D0, D,  E, F, F4)*fp.x;

	pColor.rgb = clamp(E + s1 + s2, 0.0, 1.0);

   // alpha compute

	B1  = vec3(texture(Sampler0, t1.yw).a); 
	B   = vec3(texture(Sampler0, t2.yw).a); 
	D   = vec3(texture(Sampler0, t3.xw).a); 
	E   = vec3(texture(Sampler0, t3.yw).a); 
	F   = vec3(texture(Sampler0, t3.zw).a); 
	H   = vec3(texture(Sampler0, t4.yw).a); 
	H5  = vec3(texture(Sampler0, t5.yw).a); 
	D0  = vec3(texture(Sampler0, t6.xz).a); 
	F4  = vec3(texture(Sampler0, t7.xz).a); 

	s1 = res2x(B1, B,  E, H, H5)*fp.y;
	s2 = res2x(D0, D,  E, F, F4)*fp.x;

	pColor.a = vec3(clamp(E + s1 + s2, 0.0, 1.0)).r;

    if(pColor.a < pParam1.z)
		pColor.a = 0.0;	
	
	if (pParam1.w > 0.0)
	{
		gl_FragDepth = pColor.r;
	}
}
